@@ -93,6 +93,80 @@ concentration_GW <- function(dep.var = 1, state, cache, i, j, edge, update,
 }
 
 
+#' concentration_GW_resource_covar_bin
+#' 
+#' Is there a bandwaggon effect in mobility, akin to the concentration_GW
+#' effect, but where people of group 1 only consider others of group 1
+#' in their decision to move?
+#' 
+#' @param dep.var 
+#' @param state 
+#' @param resource.attribute.index 
+#' @param cache 
+#' @param i 
+#' @param j 
+#' @param edge 
+#' @param update 
+#' @param getTargetContribution 
+#' @param alpha 
+#'
+#' @return Returns the change statistic or target statistic of the effect for 
+#' internal use by the estimation algorithm.
+#' @keywords internal
+concentration_GW_resource_covar_bin <- function(dep.var = 1, 
+                                                state,
+                                                resource.attribute.index,
+                                                cache, i, j, 
+                                                edge,
+                                                update, 
+                                                getTargetContribution = FALSE, 
+                                                alpha = 2){
+  if(alpha < 1) stop("alpha parameter in concentration_GW function must be 1 or larger")
+  
+  if(i == j) return(0)
+  
+  if(getTargetContribution){
+    
+    g_cum <- function(y, a){
+      contr <- 0
+      for(k in 0:y){
+        contr <- contr + (y-k) * exp(-log(a)*k)
+      }
+      contr - y
+    }
+    
+    nResources <- cache[[dep.var]]$resourceNetworks[[resource.attribute.index]][i, j]
+    
+    return(g_cum(y = nResources, a = alpha))
+  }
+  
+  ### calculate change statistic
+  
+  # if resource is of type 0, it cannot contribute
+  
+  if(state[[resource.attribute.index]]$data[edge] == 0){
+    return(0)
+  }
+  
+  g_mar <- function(y, a){
+    contr <- 0
+    for(k in 0:y){
+      contr <- contr + exp(-log(a)*k)
+    }
+    contr - 1
+  }
+  
+  tie_val <- cache[[dep.var]]$resourceNetworks[[resource.attribute.index]][i, j]
+  
+  if(update < 0){
+    return(update * g_mar(y = (tie_val + update), a = alpha))
+  }
+  if(update > 0){
+    return(update * g_mar(y = tie_val, a = alpha))
+  }
+}
+
+
 #' concentration_GW_dyad_covar
 #' 
 #' Are bandwagon effects (concentration) particularly prevalent between locations 
@@ -184,26 +258,30 @@ concentration_GW_dyad_covar <- function(dep.var = 1, attribute.index, state, cac
 #' @keywords internal
 concentration_prop <- function(dep.var = 1, state, cache, i, j, edge, update, 
                                getTargetContribution = FALSE){
+  ### if only one person is in the origin, 
+  ### this origin contributes 0 to the statistic
+  if(sum(cache[[dep.var]]$valuedNetwork[i,]) < 2){
+    return(0)
+  }
+  
+  ### calculate target statistic
   if(getTargetContribution){
     numerator <- 2 * (cache[[dep.var]]$valuedNetwork[i, j])^2 - 
       cache[[dep.var]]$valuedNetwork[i, j] * sum(cache[[dep.var]]$valuedNetwork[i,]) - 
       cache[[dep.var]]$valuedNetwork[i, j]
     denominator <- sum(cache[[dep.var]]$valuedNetwork[i,]) - 1
-    if(denominator <= 0) {
-      numerator <- 0
-      denominator <- 1
-    }
     return( numerator/denominator )
   }
   
   ### calculate change statistic
-  
   if(update == -1){
-    cont <- 2*(-2*cache[[dep.var]]$valuedNetwork[i, j] + 1 + sum(cache[[dep.var]]$valuedNetwork[i,]) ) /
+    cont <- 2*(-2*cache[[dep.var]]$valuedNetwork[i, j] + 1 + 
+                 sum(cache[[dep.var]]$valuedNetwork[i,]) ) /
       (sum(cache[[dep.var]]$valuedNetwork[i,]) - 1)
   }
   if(update == 1){
-    cont <- 2*(2*cache[[dep.var]]$valuedNetwork[i, j] - sum(cache[[dep.var]]$valuedNetwork[i,])) / 
+    cont <- 2*(2*cache[[dep.var]]$valuedNetwork[i, j] - 
+                 sum(cache[[dep.var]]$valuedNetwork[i,])) / 
       sum(cache[[dep.var]]$valuedNetwork[i,])
   }
   return(cont)
@@ -308,3 +386,109 @@ concentration_rankGW <- function(dep.var = 1, state, cache, i, j, edge, update,
     return(v)
   }
 }
+
+
+#' concentration_basic_squared
+#' 
+#' Is there a bandwagon effect in mobility, i.e. do mobile individuals move to 
+#' locations that are the destination of many others from their origin?
+#' 
+#' @param dep.var 
+#' @param state 
+#' @param cache 
+#' @param i 
+#' @param j 
+#' @param edge 
+#' @param update 
+#' @param getTargetContribution 
+#'
+#' @return Returns the change statistic or target statistic of the effect for 
+#' internal use by the estimation algorithm.
+#' @keywords internal
+concentration_basic_squared <- function(dep.var = 1, state, cache, i, j, edge, update, getTargetContribution = FALSE){
+  if(getTargetContribution){
+    return( sum(cache[[dep.var]]$valuedNetwork[i,j]^2 * 
+                  cache[[dep.var]]$valuedNetwork[i,]^2) )
+  } else {
+    previous_rows <- cache[[dep.var]]$valuedNetwork[i,]
+    new_rows <- previous_rows
+    new_rows[j] <- new_rows[j] + update
+    v <- (sum(new_rows^2))^2 - (sum(previous_rows^2))^2
+    return(v)
+  }
+}
+
+
+#' concentration_norm
+#' 
+#' Is there a bandwagon effect in mobility, i.e. do mobile individuals move to 
+#' locations that are the destination of many others from their origin?
+#' 
+#' @param dep.var 
+#' @param state 
+#' @param cache 
+#' @param i 
+#' @param j 
+#' @param edge 
+#' @param update 
+#' @param getTargetContribution 
+#'
+#' @return Returns the change statistic or target statistic of the effect for 
+#' internal use by the estimation algorithm.
+#' @keywords internal
+concentration_norm <- function(dep.var = 1, state, cache, i, j, edge, update, getTargetContribution = FALSE){
+  if(getTargetContribution){
+    return( cache[[dep.var]]$valuedNetwork[i,j]^2 / 
+              (sum(cache[[dep.var]]$valuedNetwork[i,]) - 1) )
+  } else {
+    if(update == -1){
+      cont <- ((cache[[dep.var]]$valuedNetwork[i,j]-1)^2 -
+               cache[[dep.var]]$valuedNetwork[i,j]^2 ) /
+        (sum(cache[[dep.var]]$valuedNetwork[i,]) - 1)
+    }
+    if(update == 1){
+      cont <- ((cache[[dep.var]]$valuedNetwork[i,j]+1)^2 -
+                 cache[[dep.var]]$valuedNetwork[i,j]^2 ) /
+        sum(cache[[dep.var]]$valuedNetwork[i,])
+    }
+    return(cont)
+  }
+}
+
+#' concentration_norm_squared
+#' 
+#' Is there a bandwagon effect in mobility, i.e. do mobile individuals move to 
+#' locations that are the destination of many others from their origin?
+#' 
+#' @param dep.var 
+#' @param state 
+#' @param cache 
+#' @param i 
+#' @param j 
+#' @param edge 
+#' @param update 
+#' @param getTargetContribution 
+#'
+#' @return Returns the change statistic or target statistic of the effect for 
+#' internal use by the estimation algorithm.
+#' @keywords internal
+concentration_norm_squared <- function(dep.var = 1, state, cache, i, j, edge, update, getTargetContribution = FALSE){
+  if(getTargetContribution){
+    return( sum(cache[[dep.var]]$valuedNetwork[i,j]^2 * cache[[dep.var]]$valuedNetwork[i,]^2 / 
+            (sum(cache[[dep.var]]$valuedNetwork[i,]) - 1)^2 ))
+  } else {
+    previous_rows <- cache[[dep.var]]$valuedNetwork[i,]
+    new_rows <- previous_rows
+    if(update == -1){
+      new_rows[j] <- new_rows[j] - 1
+      cont <- ((sum(new_rows^2 / (sum(cache[[dep.var]]$valuedNetwork[i,]) - 1)))^2 - 
+               (sum(previous_rows^2 / (sum(cache[[dep.var]]$valuedNetwork[i,]) - 1)))^2) 
+    }
+    if(update == 1){
+      new_rows[j] <- new_rows[j] + 1
+      cont <- ((sum(new_rows^2 / (sum(cache[[dep.var]]$valuedNetwork[i,]))))^2 - 
+               (sum(previous_rows^2 / (sum(cache[[dep.var]]$valuedNetwork[i,]))))^2) 
+    }
+    return(cont)
+  }
+}
\ No newline at end of file
