@@ -13,17 +13,11 @@
 "myState"
 
 
-#' @rdname myOutcomeObjects
-#' @usage NULL
-#' @format `myCache`
-#' A "list" created by the function [createWeightedCache()].
-"myCache"
-
-
 #' @rdname myOutcomeObjects
 #' @usage NULL
 #' @format `myEffects`
-#' An object of class "effectsList.monan" created by the function [createEffectsObject()].
+#' An object of class "effectsList.monan" created by the function [createEffectsObject()]
+#' or [createEffects()].
 "myEffects"
 
 
